<?php

namespace PlaNext\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\FilterCriteria;

class PedidoAlteracaoCardapioController {
    
    protected $crudService;
    
    public function __construct($connection, $environment, $service, $crudService) {
    $this->connection   = $connection;
    $this->environment  = $environment;
    $this->service      = $service;
    $this->crudService  = $crudService;
  }
    
    public function getInicialPedidoAlteracao(Request\Row $request, Response $response) {
        $data = array(
              'NRORG' => $this->environment->getNrOrg(),
              'CDOPERADOR' => $this->environment->getCdOperador(),
              "FILIAL_LOGADA" => $this->environment->getCdFilial()
        );
        $response->addDataSet(new DataSet("filtroInicial", $data));
    }
    
    public function insereNovoPedido(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $row = $request->getParameters();
            $row['NRORG'] = $this->environment->getNrOrg();
            $row['CDOPERADOR'] = $this->environment->getCdOperador();
            
            $row['NRPEDALTCARD'] = $this->service->retornaUltimoNRPEDALT($row['NRORG']);
            $valorDataMes = $row['DTMESMODAL'];
            list($mes, $ano) = explode('-', $valorDataMes);
            $dataFormatada = '01-'. $mes. '-'. $ano;
            $row['DTMESMODAL'] = $dataFormatada;
            
            $row['CDPEDALTCARD'] = str_pad($row['NRPEDALTCARD'], 10, 0, STR_PAD_LEFT);
            
            $this->service->insertNovoPedido($row);
            $response->addDataSet(new DataSet("valoresInseridos", $row));
        }catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function deletaPedido(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $parameters = $request->getParameters();
            $row = $parameters['rowDelete'];
            $this->service->deletePedidoService($row);
            $response->addDataSet(new DataSet("valoresDeletados", $row));
        }catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function atualizaPedido(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $parameters = $request->getParameters();
            $valoresUpdate = $parameters['valoresUpdate'];
            $row = $valoresUpdate['row'];
            $row['DSJUSTICATIVA'] = $valoresUpdate['justificativa'];
            $this->service->atualizaPedidoService($row);
            $rowsAtualizadas = array(
                $row['DSJUSTICATIVA'],
                $row['NRPEDALTCARD'],
                $row['CDPEDALTCARD'],
                $row['CDFILIAL']
            );
            $response->addDataSet(new DataSet("valoresAtualizados", $rowsAtualizadas));
        }catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function carregaDadosCardapio(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $parameters = $request->getParameters();
            $row = $parameters['currentRow'];
            $dataColumns = $this->service->carregaColunasCardapioService($row);
            $response->addDataSet(new DataSet('ColunasCardapio', $dataColumns));
            
            $dataPratos = $this->service->retornaPratosCardapio($row);
            $response->addDataSet(new DataSet('PratosCardapio', $dataPratos));
            
            $dataComensais = $this->service->retornaComensaisCardapio($row);
            $response->addDataSet(new DataSet('ComensaisCardapio', $dataComensais));
            
            $changedDiners = $this->service->returnsChangedDiners($row);
            $changedDishes = $this->service->returnsChangedDishes($row);
            $response->addDataSet(new DataSet('ChangedDiners', $changedDiners));
            $response->addDataSet(new DataSet('ChangedDishes', $changedDishes));
        }catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaExisteCardapio(Request\Row $request, Response $response) {
        try{
            $row = $request->getParameters();
            $dataVerifica = $this->service->verificaExisteCardapio($row);
            $response->addDataSet(new DataSet("verificaExisteCardapio", $dataVerifica));
        }catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaTurnosPontoEntrega(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $row = $request->getParameters()['currentRowGrid'];
            $dataTurnos = $this->service->verificaTurnosCardaDia($row);
            $response->addDataSet(new DataSet("turnosCardaDia", $dataTurnos));
            $dataPontosEntrega = $this->service->verificaPontoEntrega($row);
            if($dataPontosEntrega === []) {
                $dataPontosEntrega = $this->service->retornaCenario($row);
            }
            $response->addDataSet(new DataSet("pontosDeEntrega", $dataPontosEntrega));
        }catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function alteraComensaisTurno(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $row = $request->getParameters();
            $row['NRORG'] = $this->environment->getNrOrg();
            $row['CDOPERADOR'] = $this->environment->getCdOperador();
            $nrCardapio = $this->service->retornaNrCardapio($row['valoresUpdate']['row']);
            if (!empty($nrCardapio)) {
                $row['NRCARDAPIO'] = $nrCardapio[0]['NRCARDAPIO'];
            } else {
                $row['NRCARDAPIO'] = null;
            }
            $dataPedidoAlteraçãoExistente = $this->service->changedDinerDtMenu($row);
            if(count($dataPedidoAlteraçãoExistente) > 0){
                $row['NRPEDALTDIA'] = $dataPedidoAlteraçãoExistente[0]['NRPEDALTCARDDIA'];
                $this->service->alteraComensaisTurnoService($row);
            } else {
                $row['NRPEDALTCARDDIA'] = $this->service->retornaNRPEDALTCARDDIA($row['NRORG']);
                $this->service->insereComensaisTurnoService($row);
            }
            $this->connection->commit();
        }catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function changedDish(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $row = $request->getParameters();
            $row['NRORG'] = $this->environment->getNrOrg();
            $row['CDOPERADOR'] = $this->environment->getCdOperador();
            $nrCardapio = $this->service->retornaNrCardapio($row['valoresUpdate']['row']);
            if (!empty($nrCardapio)) {
                $row['NRCARDAPIO'] = $nrCardapio[0]['NRCARDAPIO'];
            } else {
                $row['NRCARDAPIO'] = null;
            }
            $dataPedidoAlteraçãoExistente = $this->service->changedDishDtMenu($row);
            if(count($dataPedidoAlteraçãoExistente) > 0){
                $this->service->updateChangedDish($row);
            } else {
                $row['NRPEDALTCARDCOMP'] = $this->service->returnNRPEDALTCARDCOMP($this->environment->getNrOrg());
                $this->service->insertChangedDish($row);
            }
            $this->connection->commit();
        }catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function checksWithdrawalsEffectives($row) {
        $withdrawalMenu = $this->service->withdrawalMenu($row);
        if($withdrawalMenu[0]['RETIRADAS'] != 0) {
            throw new \Exception('plaNext::errorWithdrawals');
        }
        $effectivesMenu = $this->service->effectivesMenu($row);
        if($effectivesMenu[0]['EFETIVOS'] != 0) {
            throw new \Exception('plaNext::errorEffectives');
        }
    }
    
    public function completeOrder(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $this->connection->beginTransaction();
        try{
            $changedDiners = $this->service->returnsChangedDiners($row['CURRENTROW']);
            if(count($changedDiners) === 0) {
                $changedDishes = $this->service->returnsChangedDishes($row['CURRENTROW']);
                if(count($changedDishes) === 0) {
                    throw new \Exception('plaNext::noChangedMenu');
                }
            }
            $this->checksWithdrawalsEffectives($row['CURRENTROW']);
            $operator = $this->environment->getCdOperador();
            if($row['EXTRAREQUEST'] === "S" || ($row['EXTRAREQUEST'] === "N" && count($changedDiners) != 0)) {
                $this->service->completeOrder($row['CURRENTROW'], $row['EXTRAREQUEST'], $operator);
            } else if ($row['EXTRAREQUEST'] === "N" && count($changedDiners) === 0) {
                $row['JUSTIFICATION'] = null;
                $this->service->approve($row, $operator);
            }
            $this->connection->commit();
        }catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function requestAgain(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $row = $request->getRow();
            $operator = $this->environment->getCdOperador();
            $this->service->requestAgain($row, $operator);
            $this->connection->commit();
        }catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function approveDisapprove(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $row = $request->getRow();
            $operator = $this->environment->getCdOperador();
            if ($row['APPROVALPAGE']) {
                if($row['APPROVE'] === 'A') {
                    $this->service->approve($row, $operator);
                } else {
                    $this->service->disapprove($row, $operator);
                }
            } else {
                if($row['APPROVE'] === 'A') {
                    $this->service->approveGO($row, $operator);
                } else {
                    $this->service->disapproveGO($row, $operator);
                }
            }
            $this->connection->commit();
        }catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}