import {
  Http,
  Metadata,
  IDictionary,
  IEventParam,
} from '@zeedhi/core';
import {
  Select,
  Card,
  Grid,
  Form,
  Modal,
  Button,
  Divider,
  Number as NumberInput,
  LoadingService,
  GridColumnEditable,
  GridEditable,
  ModalService,
  AlertService,
  DialogService,
  Textarea,
  IComponentRender,
} from '@zeedhi/common';

export class PedidoAlteracaoCardapioController {
  public newRow = false;

  public isInlineEditing = false;

  private modalIncluir!: Modal;

  private modalAlteracao!: Modal;

  private modalResumo!: Modal;

  private modalAlteraCellComensais!: Modal;

  private modalAlteraPratos!: Modal;

  private approveDisapproveModal!: Modal;

  private completeOrderModal!: Modal;

  public checkModeApproval() {
    const page = Metadata.getInstance<Card>('pedidoAlteracaoCardapio');
    return (page.cssClass === 'APPROVAL');
  }

  public visibleButtonsApproval() {
    const addButton = Metadata.getInstance<Button>('botaoIncluirPedido');
    const deleteOrderButton = Metadata.getInstance<Button>('botaoExcluirPedido');
    const divisor1 = Metadata.getInstance<Divider>('divisor2');
    const concludeButton = Metadata.getInstance<Button>('botaoConcluirPedido');
    const requestButton = Metadata.getInstance<Button>('botaoSolicitarNovamente');
    const divisor2 = Metadata.getInstance<Divider>('divisor3');
    const calcelButton = Metadata.getInstance<Button>('cancelButton');
    const saveButton = Metadata.getInstance<Button>('saveButton');
    addButton.isVisible = false;
    deleteOrderButton.isVisible = false;
    divisor1.isVisible = false;
    concludeButton.isVisible = false;
    requestButton.isVisible = false;
    divisor2.isVisible = false;
    calcelButton.isVisible = false;
    saveButton.isVisible = false;
  }

  public async getInicialPedidoAlteracao() {
    Http.post('/getInicialPedidoAlteracao').then((response) => {
      const data = response.data.dataset.filtroInicial;
      localStorage.setItem('CDOPERADOR', data.CDOPERADOR);
      localStorage.setItem('NRORG', data.NRORG);
      localStorage.setItem('FILIAL_FILTRO', data.FILIAL_LOGADA);
      this.createApproveDisapproveModal();
      this.createCompleteOrderModal();
    });
  }

  public destroyModal() {
    this.approveDisapproveModal.destroy();
    this.completeOrderModal.destroy();
  }

  public idSolExtraAction({ row }: any) {
    return row.IDSOLEXTRA === 'S';
  }

  public idSolExtraActionOff({ row }: any) {
    return row.IDSOLEXTRA === 'N';
  }

  public filtroFilialFilioper() {
    const filialFiltro = Metadata.getInstance<Select>('FILIAL_FILTRO');
    const cdOperador = localStorage.getItem('CDOPERADOR');
    const nrOrg = localStorage.getItem('NRORG');
    const filter = {
      CDOPERADOR: cdOperador,
      NRORG: nrOrg,
    };
    filialFiltro.datasource.filter = filter;
    filialFiltro.datasource.get();
    filialFiltro.value = localStorage.getItem('FILIAL_LOGADA');
  }

  public configuraGridAlteracao() {
    const cardGridAlteracao = Metadata.getInstance<Card>('cardGrid');
    const gridAlteracao = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const cardForms = Metadata.getInstance<Card>('cardForms');
    const filialFiltro = Metadata.getInstance<Select>('FILIAL_FILTRO').value;
    const filtro = {
      CDFILIAL: filialFiltro,
    };
    if (this.checkModeApproval()) {
      this.visibleButtonsApproval();
    }
    gridAlteracao.datasource.filter = filtro;
    gridAlteracao.datasource.get().then(() => {
      cardGridAlteracao.isVisible = true;
      cardForms.isVisible = true;
    });
  }

  public returnGrid(component: IDictionary) {
    let grid: IDictionary;
    if (component.type === 'button') grid = component.parent as IDictionary;
    else grid = component;
    return grid;
  }

  public async criaModalIncluir() {
    this.modalIncluir = await ModalService.createFromJson('/modals/novoPedido', true);
  }

  public abreModalIncluir() {
    const grid: Grid = Metadata.getInstance('gridAlteracaoCardapio');
    if (grid) {
      this.criaModalIncluir().then(() => {
        this.modalIncluir.show();
      });
    }
  }

  public fechaModalIncluir() {
    this.modalIncluir.hide();
    this.modalIncluir.destroy();
  }

  public filtroFilialServico({ component }: IEventParam<Select>) {
    const cdFilial = Metadata.getInstance<Select>('FILIAL_FILTRO').value;
    const conditions = {
      CDFILIAL: cdFilial,
    };
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public confirmaNovoPedido() {
    const form = Metadata.getInstance<Form>('containerModalForm');
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const textAreaJustificativa = Metadata.getInstance<Textarea>('justificativaPedido');
    const row = form.value;
    row.CDFILIAL = Metadata.getInstance<Select>('FILIAL_FILTRO').value;
    const [mes, ano] = row.DTMESMODAL.split('-');
    const dataDia = `01/${mes}/${ano}`;
    row.DTMESCARD = dataDia;
    LoadingService.show();
    Http.post('/verificaExisteCardapio', row).then((response) => {
      const data = response.data.dataset.verificaExisteCardapio;
      const existeCardapio = data[0].EXISTE_CARDAPIO;
      if (form.validate() && existeCardapio !== 0) {
        LoadingService.show();
        Http.post('/insereNovoPedido', row).then(() => {
          this.fechaModalIncluir();
          grid.datasource.get().then(() => {
            grid.reload();
          });
        }).finally(() => {
          LoadingService.hide();
          form.clearForm();
          textAreaJustificativa.value = '';
          textAreaJustificativa.readonly = true;
        });
      } else if (existeCardapio === 0) {
        DialogService.show({
          name: 'error-dialog',
          type: 'error',
          title: 'plaNext::naoIncluiuPedido',
          text: 'plaNext::naoExisteCardapio',
        });
      }
    }).finally(() => {
      LoadingService.hide();
    });
  }

  private configuraAlert(nameAlert, textAlert, colorAlert) {
    return {
      name: nameAlert,
      text: textAlert,
      color: colorAlert,
      top: true,
      left: false,
      timeout: 3000,
      buttonsSlot: [
        {
          name: 'undo-button',
          component: 'ZdButton',
          icon: true,
          iconName: 'close',
          color: 'white',
          events: {
            click: () => { AlertService.hide(); },
          },
        },
      ],
    };
  }

  public deletaPedido({ component }: IDictionary) {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const row = grid.datasource.currentRow;
    if (!row.DTINCLUSAO) {
      AlertService.show(this.configuraAlert('alertError', 'plaNext::nenhumRegSelecionado', 'error'));
    } else if (row.IDSITUACAO === 'E') {
      DialogService.show({
        name: component.name,
        type: 'warning',
        title: 'plaNext::atencao',
        text: 'plaNext::desejaExcluir',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'plaNext::sim',
            events: { click: () => this.callDeleteTree() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'plaNext::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    } else {
      AlertService.show(this.configuraAlert('alertError', 'plaNext::noOrderInProgress', 'error'));
    }
  }

  private callDeleteTree() {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const row = grid.datasource.currentRow;
    const textAreaJustificativa = Metadata.getInstance<Textarea>('justificativaPedido');
    const rowDelete = {
      CDPEDALTCARD: row.CDPEDALTCARD,
      CDFILIAL: row.CDFILIAL,
    };
    LoadingService.show();
    Http.post('/deletaPedido', { requestType: 'Row', rowDelete }).then((response) => {
      if (response) {
        AlertService.show(this.configuraAlert('alert', 'plaNext::excluidoComSucesso', 'success'));
        grid.datasource.get().then(() => {
          grid.reload();
        });
      }
    }).catch((error: string) => {
      AlertService.show(this.configuraAlert('alertError', error, 'error'));
    }).finally(() => {
      LoadingService.hide();
      DialogService.hide();
      textAreaJustificativa.value = '';
      textAreaJustificativa.readonly = true;
    });
  }

  public configuraCampoJustificativa() {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const textAreaJustificativa = Metadata.getInstance<Textarea>('justificativaPedido');
    const { currentRow } = grid.datasource;
    textAreaJustificativa.readonly = this.checkModeApproval() || false;
    textAreaJustificativa.value = currentRow.DSJUSTICATIVA;
  }

  private configuraActions(actionPadrao, actionJustificativa) {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const toolbar = grid.toolbarSlot;
    toolbar.forEach((e: IComponentRender) => {
      if (e.name === 'incluirPedido') e.children[0].disabled = actionPadrao;
      if (e.name === 'excluirPedido') e.children[0].disabled = actionPadrao;
      if (e.name === 'concluirPedido') e.children[0].disabled = actionPadrao;
      if (e.name === 'solicitarNovamente') e.children[0].disabled = actionPadrao;
      if (e.name === 'aprovacaoReprovacaoPedido') e.children[0].disabled = actionPadrao;
      if (e.name === 'cancelButton') e.disabled = actionJustificativa;
      if (e.name === 'saveButton') e.disabled = actionJustificativa;
    });
  }

  public editandoJustificativa() {
    const textAreaJustificativa = Metadata.getInstance<Textarea>('justificativaPedido');
    if (textAreaJustificativa.readonly === false) {
      this.configuraActions(true, false);
    }
  }

  public cancelaEdicaoJustificativa() {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const buttonInclusaoPedido = Metadata.getInstance<Button>('botaoIncluirPedido');
    const textAreaJustificativa = Metadata.getInstance<Textarea>('justificativaPedido');
    this.configuraActions(false, true);
    buttonInclusaoPedido.disabled = false;
    textAreaJustificativa.value = '';
    textAreaJustificativa.readonly = true;
    grid.datasource.get();
  }

  public updateJustificativa() {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const textAreaJustificativa = Metadata.getInstance<Textarea>('justificativaPedido');
    const { currentRow } = grid.datasource;
    const valorJustificativa = textAreaJustificativa.value;
    const valoresUpdate = {
      row: currentRow,
      justificativa: valorJustificativa,
    };
    LoadingService.show();
    Http.post('/atualizaPedido', { requestType: 'Row', valoresUpdate }).then((response) => {
      if (response) {
        AlertService.show(this.configuraAlert('alert', 'plaNext::atualizadoComSucesso', 'success'));
      }
    }).catch((error: string) => {
      AlertService.show(this.configuraAlert('alertError', error, 'error'));
    }).finally(() => {
      LoadingService.hide();
      const buttonInclusaoPedido = Metadata.getInstance<Button>('botaoIncluirPedido');
      this.configuraActions(false, true);
      buttonInclusaoPedido.disabled = false;
      textAreaJustificativa.value = '';
      textAreaJustificativa.readonly = true;
      grid.datasource.get();
    });
  }

  public async criaModalAlteracao() {
    this.modalAlteracao = await ModalService.createFromJson('/modals/alteracaoCardapio', true);
  }

  public abreModalAlteracao({ row }) {
    const grid: Grid = Metadata.getInstance('gridAlteracaoCardapio');
    grid.datasource.currentRow = row;
    this.criaModalAlteracao().then(() => {
      this.modalAlteracao.show();
    });
  }

  public fechaModalAlteracao() {
    this.modalAlteracao.hide();
    this.modalAlteracao.destroy();
  }

  private getAllDaysInMonth(dateString: string): string[] {
    const [, month, year] = dateString.split('/').map(Number);
    const daysInMonth = new Date(year, month, 0).getDate();
    const days: string[] = [];
    const weekNames: string[] = [
      'DOM',
      'SEG',
      'TER',
      'QUA',
      'QUI',
      'SEX',
      'SAB',
    ];
    for (let dayIndex = 1; dayIndex <= daysInMonth; dayIndex += 1) {
      const dayOfWeek = new Date(year, month - 1, dayIndex).getDay();
      const formattedDay = `${weekNames[dayOfWeek]} ${String(dayIndex).padStart(2, '0')}`;
      days.push(formattedDay);
    }
    return days;
  }

  // MODAL DE CARDAPIO - CONFIGURAÇÃO E CRIAÇÃO
  private comensais = [];

  public configuraAlteracoesCardapio(row) {
    return Http.post('/verificaAlteracaoCardapio', { requestType: 'Row', row }).then((response) => {
      if (response) {
        if (this.dataCardapioAlteracao.length > 0) {
          this.dataCardapioAlteracao = [];
        }
        if (this.dataAlteracoesExistentes.length > 0) {
          this.dataAlteracoesExistentes = [];
        }
        this.dataCardapioAlteracao.push(response.data.dataset.dataCardapioAlteracao);
        this.dataAlteracoesExistentes.push(response.data.dataset.alteracoesExistentes);
      }
      return this.dataCardapioAlteracao;
    }).catch((error: string) => {
      AlertService.show(this.configuraAlert('alertError', error, 'error'));
    });
  }

  private dataCardapioAlteracao = [];

  private dataAlteracoesExistentes = [];

  public configureMenu() {
    this.configureMenuChanged();
  }

  public configureMenuChanged(newChanged = false) {
    const grid: Grid = Metadata.getInstance('gridAlteracaoCardapio');
    const { currentRow } = grid.datasource;
    const [data] = currentRow.DTMESCARD.split(' ');
    currentRow.DTMESCARD = data;
    currentRow.DTREFECNFMES = data;
    const gridMenu : GridEditable = Metadata.getInstance('tableCardapio');
    const dateMonth = this.getAllDaysInMonth(currentRow.DTREFECNFMES);
    Http.post('/carregaDadosCardapio', { requestType: 'Row', currentRow }).then((response) => {
      const responseData = response.data.dataset;
      const changedDishes = responseData.ChangedDishes;
      const changedDiners = responseData.ChangedDiners;
      const arrayColumns = responseData.ColunasCardapio;
      const arrayDiners = responseData.ComensaisCardapio;
      const arrayDishes = response.data.dataset.PratosCardapio;
      const gridColumns = gridMenu.columns;
      if (!newChanged) {
        this.createColumns(gridColumns, gridMenu, arrayColumns);
      }
      this.fillMenu(gridMenu, changedDiners, changedDishes, arrayDiners, dateMonth, gridColumns, arrayDishes, currentRow);
    });
  }

  public createColumns(gridColumns, gridMenu, arrayColumns) {
    gridColumns.push(new GridColumnEditable(
      {
        name: 'DIASCARDAPIO',
        label: 'plaNext::diaUpper',
      },
    ));
    gridColumns.push(new GridColumnEditable(
      {
        name: 'COMENSAISCARDAPIO',
        label: 'plaNext::comensaisUpper',
        conditions: {
          cssClass: '{{PedidoAlteracaoCardapioController.getConditionalCssClass}}',
        },
      },
    ));
    arrayColumns.forEach((item) => {
      const nmTipoPrato = item.NMTIPOPRAT.replace(/\s+/g, '');
      if (item.DSCOMPLTPMES === null) {
        /* eslint-disable prefer-template */
        gridColumns.push(new GridColumnEditable(
          {
            name: nmTipoPrato + item.NRORDCNFMES,
            label: item.NMTIPOPRAT,
            componentProps: {
              name: 'NRSEQPRATCAR' + item.NRORDCNFMES,
              component: 'ZdText',
              text: item.NRORDCNFMES,
            },
            conditions: {
              cssClass: '{{PedidoAlteracaoCardapioController.getConditionalCssClass}}',
            },
          },
        ));
      } else if (item.DSCOMPLTPMES !== null) {
        gridColumns.push(new GridColumnEditable(
          {
            name: nmTipoPrato + item.NRORDCNFMES,
            label: item.NMTIPOPRAT + ' (' + item.DSCOMPLTPMES + ')',
            orderColumn: item.NRORDCNFMES,
            conditions: {
              cssClass: '{{PedidoAlteracaoCardapioController.getConditionalCssClass}}',
            },
          },
        ));
        /* eslint-enable prefer-template */
      }
    });
  }

  public fillMenu(gridMenu, changedDiners, changedDishes, arrayDiners, dateMonth, gridColumns, arrayDishes, currentRow) {
    if (this.comensais.length > 0) {
      this.comensais = [];
    }
    const arrayColumns = [];
    gridMenu.datasource.updateData([]);
    dateMonth.forEach((e) => {
      gridColumns.forEach((g) => {
        if (g.name !== 'DIASCARDAPIO' && g.name !== 'COMENSAISCARDAPIO') {
          if (arrayColumns.includes(g.name) === false) {
            arrayColumns.push(
              {
                name: g.name,
                label: g.label,
              },
            );
          }
        }
      });
      type TipoColunas = typeof arrayColumns[number];
      type TipoPrato = { [key in TipoColunas]?: string };
      type TipoEspecial = {
        DIASCARDAPIO?: string;
        COMENSAISCARDAPIO?: string;
      };
      type NewRow = TipoPrato & TipoEspecial;
      const newRow: NewRow = {} as NewRow;
      newRow.DIASCARDAPIO = e;
      const dayWeek = e.substring(4);
      newRow.CDFILIAL = currentRow.CDFILIAL;
      this.fillDiners(arrayDiners, dayWeek, changedDiners, newRow);
      this.fillDishes(changedDishes, dayWeek, arrayDishes, arrayColumns, newRow);
      gridMenu.addNewRow(newRow);
    });
  }

  public fillDiners(arrayDiners, dayWeek, changedDiners, newRow) {
    const [changedDiner] = changedDiners.filter((item) => (item.DTCARDAPIO.split('/')[0] === dayWeek));
    arrayDiners.forEach((diner) => {
      const dayDiner = diner.DTCARDAPIO.split('/')[0];
      if (dayDiner === dayWeek && changedDiner) {
        this.fillChangedDiner(diner, changedDiner, newRow);
      } else if (dayDiner === dayWeek) {
        newRow.COMENSAISCARDAPIO = diner.QTCOMESTICAR;
      } else if (changedDiner) {
        newRow.CHANGED_COLUMNS = 'changedRecordMenu';
        newRow.COMENSAISCARDAPIO = changedDiner.QTCLIESTICAR + changedDiner.QTDOAESTICAR + changedDiner.QTFUNESTICAR;
      }
    });
    this.comensais.push(arrayDiners);
  }

  public fillDishes(changedDishes, dayWeek, arrayDishes, arrayColumns, newRow) {
    const arrayColumnsChangedDishes = [];
    arrayColumns.forEach((c) => {
      const { name } = c;
      const match = name.match(/\d+$/);
      const numOrder = match ? match[0] : '';
      const [changedInArray] = arrayColumnsChangedDishes.filter((item) => (
        item === c.label
      ));
      const [changedDish] = changedDishes.filter((item) => (
        item.DTCARDAPIO.split('/')[0] === dayWeek && item.NRSEQPRATCAR.toString() === numOrder
      ));
      arrayDishes.forEach((dish) => {
        const diaDoPrato = dish.DTCARDAPIO.substring(0, 2);
        const pratoTipo = dish.NMTIPOPRAT;
        const itemOrder = dish.NRSEQPRATCAR.toString();
        if (diaDoPrato === dayWeek && changedDish && !changedInArray) {
          const itemOrderChanged = changedDish.NRSEQPRATCAR.toString();
          if (c.label === pratoTipo && numOrder === itemOrder && numOrder === itemOrderChanged) {
            dish.CDPRATO = changedDish.CDPRATO;
            dish.NMPRATO = changedDish.NMPRATO;
            dish.VRPECONSCARD = changedDish.VRPECONSCARD;
            arrayColumnsChangedDishes.push(dish.NMTIPOPRAT);
            newRow[c.name] = `${Number(changedDish.VRPECONSCARD).toFixed(2)}% ${dish.NMPRATO}`;
            c.labelAlterada = `${c.label.replace(/\s+/g, '')}${itemOrder}CD`;
            newRow[c.labelAlterada] = dish.CDPRATO;
            newRow.NRSEQPRATCAR = dish.NRSEQPRATCAR;
            if (dish.CHANGED_COLUMNS_DISHES) {
              newRow.CHANGED_COLUMNS_DISHES = arrayColumnsChangedDishes;
            }
          }
        } else if (diaDoPrato === dayWeek) {
          if (c.label === pratoTipo && numOrder === itemOrder) {
            newRow[c.name] = `${Number(dish.VRPECONSCARD).toFixed(2)}% ${dish.NMPRATO}`;
            c.labelAlterada = `${c.label.replace(/\s+/g, '')}${itemOrder}CD`;
            newRow[c.labelAlterada] = dish.CDPRATO;
            newRow.NRSEQPRATCAR = dish.NRSEQPRATCAR;
          }
        }
      });
      if (changedDish && !changedInArray) {
        const itemOrderChanged = changedDish.NRSEQPRATCAR.toString();
        if (numOrder === itemOrderChanged) {
          arrayColumnsChangedDishes.push(c.label);
          newRow[c.name] = `${Number(changedDish.VRPECONSCARD).toFixed(2)}% ${changedDish.NMPRATO}`;
          c.labelAlterada = `${c.label.replace(/\s+/g, '')}${numOrder}CD`;
          newRow[c.labelAlterada] = changedDish.CDPRATO;
          newRow.NRSEQPRATCAR = changedDish.NRSEQPRATCAR;
          if (arrayColumnsChangedDishes && arrayColumnsChangedDishes.length) {
            newRow.CHANGED_COLUMNS_DISHES = arrayColumnsChangedDishes;
          }
        }
      }
    });
  }

  public fillChangedDiner(diner, changedDiner, newRow) {
    diner.QTCLIESTICAR = changedDiner.QTCLIESTICAR;
    diner.QTDOAESTICAR = changedDiner.QTDOAESTICAR;
    diner.QTFUNESTICAR = changedDiner.QTFUNESTICAR;
    diner.QTCOMESTICAR = changedDiner.QTCLIESTICAR + changedDiner.QTDOAESTICAR + changedDiner.QTFUNESTICAR;
    diner.QTESTITURNO1 = changedDiner.QTESTITURNO1;
    diner.QTESTITURNO2 = changedDiner.QTESTITURNO2;
    diner.QTESTITURNO3 = changedDiner.QTESTITURNO3;
    diner.QTESTITURNO4 = changedDiner.QTESTITURNO4;
    newRow.CHANGED_COLUMNS = 'changedRecordMenu';
    newRow.COMENSAISCARDAPIO = diner.QTCOMESTICAR;
  }

  public getConditionalCssClass({ row, column }: IDictionary<any>) {
    if (column.name === 'COMENSAISCARDAPIO' && row.CHANGED_COLUMNS) {
      return row.CHANGED_COLUMNS;
    }
    if (row.CHANGED_COLUMNS_DISHES) {
      const [dish] = row.CHANGED_COLUMNS_DISHES.filter((item) => (item === column.label));
      if (dish) {
        return 'changedRecordMenu';
      }
    }
    return '';
  }

  // MODAL DE RESUMO - CONFIGURAÇÃO E CRIAÇÃO
  public async criaModalResumo() {
    this.modalResumo = await ModalService.createFromJson('/modals/resumoAlteracaoCardapio', true);
  }

  public abreModalResumo() {
    const grid: Grid = Metadata.getInstance('tableCardapio');
    const { datasource } = grid;
    if (datasource) { // validar verificação caso tenha alteração no resumo do cardapio.
      this.criaModalResumo().then(() => {
        this.modalResumo.show();
      });
    } else {
      AlertService.show(this.configuraAlert('alertError', 'plaNext::nenhumRegSelecionado', 'error'));
    }
  }

  public fechaModalResumo() {
    this.modalResumo.hide();
    this.modalResumo.destroy();
  }

  // MODAL DE COMENSAIS - CONFIGURAÇÃO E CRIAÇÃO
  public async criaModalAlteraCellComensais() {
    this.modalAlteraCellComensais = await ModalService.createFromJson('/modals/alteraCellComensais', true);
  }

  private obterValorDiaSemana(string) {
    const diasDaSemana = {
      DOM: 0,
      SEG: 1,
      TER: 2,
      QUA: 3,
      QUI: 4,
      SEX: 5,
      SAB: 6,
    };
    const dia = string.substring(0, 3);
    return diasDaSemana[dia] !== undefined ? diasDaSemana[dia] : -1;
  }

  // MODAL DE COMENSAIS - CONFIGURAÇÃO
  public configuraCelulaComensais(row) {
    const currentRow = row;
    const grid: Grid = Metadata.getInstance('gridAlteracaoCardapio');
    const currentRowGrid = grid.datasource.currentRow;
    const dtMesCardapio = currentRowGrid.DTMESCARD.substring(2, 10);
    const diaCardapio = currentRow.DIASCARDAPIO.substring(4);
    currentRowGrid.DTDIACARD = diaCardapio + dtMesCardapio;
    const diaDaSemana = this.obterValorDiaSemana(currentRow.DIASCARDAPIO);
    currentRowGrid.DIADASEMANA = diaDaSemana;
    Http.post('/turnosPontoEntrega', { requestType: 'Row', currentRowGrid }).then((response) => {
      const comensaisArray = this.comensais[0];
      const turnosPontoEntregaArray = [];
      const turnos = response.data.dataset.turnosCardaDia;
      const pontoEntrega = response.data.dataset.pontosDeEntrega;
      const formTurno = Metadata.getInstance<Form>('formAlteraTurnos');
      const formPontoEntrega = Metadata.getInstance<Form>('formAlteraPontoEntrega');
      const clientesComensais = Metadata.getInstance<NumberInput>('clientesAlteraComensais');
      turnosPontoEntregaArray.push({
        turnos,
      });
      turnosPontoEntregaArray.push({
        pontoEntrega,
      });
      if (turnos.length > 0) {
        formTurno.isVisible = true;
      }
      if (pontoEntrega.length > 0) {
        formPontoEntrega.isVisible = false;
      }
      if (formPontoEntrega.isVisible === false) {
        clientesComensais.readonly = false;
      } else {
        clientesComensais.readonly = true;
      }
      const turnosArray = [];
      const pontoEntregaArray = [];
      turnosPontoEntregaArray.forEach((item) => {
        if (item.turnos) {
          turnosArray.push(...item.turnos);
        }
        if (item.pontoEntrega) {
          pontoEntregaArray.push(...item.pontoEntrega);
        }
      });
      comensaisArray.forEach((item) => {
        if (item.DTCARDAPIO === currentRowGrid.DTDIACARD) {
          const funcionariosComensais = Metadata.getInstance<NumberInput>('funcionariosAlteraComensais');
          const doacoesComensais = Metadata.getInstance<NumberInput>('doacoesAlteraComensais');
          const totalComensais = Metadata.getInstance<NumberInput>('totalAlteraComensais');
          clientesComensais.value = item.QTCLIESTICAR;
          funcionariosComensais.value = item.QTFUNESTICAR;
          doacoesComensais.value = item.QTDOAESTICAR;
          totalComensais.value = item.QTCOMESTICAR;
          if (item.ALTERACAOATIVA) {
            Object.keys(item).forEach((key) => {
              if (key.startsWith('QTESTITURNO') && item[key] !== null) {
                const turnoNumber = key.slice(-1);
                turnosArray.forEach((turno) => {
                  if (turno.CDTURNO === turnoNumber) {
                    turno.COMENSAIS = item[key];
                  }
                });
              }
            });
          }
        }
      });
      const qtdeTurnos = turnosArray.length;
      const colsGrid = 12 / qtdeTurnos;
      turnosArray.forEach((item) => {
        const child = {
          name: item.NMTURNO + item.CDTURNO,
          component: 'ZdNumber',
          label: item.NMTURNO,
          value: item.COMENSAIS,
          grid: {
            cols: colsGrid,
          },
          mask: {
            decimalPlaces: 0,
          },
        };
        formTurno.addChild(child);
      });
      pontoEntregaArray.forEach((item) => {
        const child = {
          name: item.NMDESTFILI + item.CDDESTFILI,
          component: 'ZdNumber',
          label: item.NMDESTFILI,
          value: item.QTCOMESTICLI,
          mask: {
            decimalPlaces: 0,
          },
        };
        formPontoEntrega.addChild(child);
      });
    });
    if (currentRow) {
      this.criaModalAlteraCellComensais().then(() => {
        this.modalAlteraCellComensais.show();
      });
    }
  }

  public fechaModalAlteraCellComensais() {
    this.modalAlteraCellComensais.hide();
    this.modalAlteraCellComensais.destroy();
  }

  private rowComensais = {
    clientesComensais: 0,
    funcionariosComensais: 0,
    doacoesComensais: 0,
    totalComensais: 0,
  };

  public configuraCamposComensais() {
    const clientesComensais = Metadata.getInstance<NumberInput>('clientesAlteraComensais');
    const funcionariosComensais = Metadata.getInstance<NumberInput>('funcionariosAlteraComensais');
    const doacoesComensais = Metadata.getInstance<NumberInput>('doacoesAlteraComensais');
    const totalComensais = Metadata.getInstance<NumberInput>('totalAlteraComensais');
    if (clientesComensais.value + funcionariosComensais.value + doacoesComensais.value !== totalComensais.value) {
      totalComensais.value = clientesComensais.value + funcionariosComensais.value + doacoesComensais.value;
    }
    this.rowComensais = {
      clientesComensais: clientesComensais.value,
      funcionariosComensais: funcionariosComensais.value,
      doacoesComensais: doacoesComensais.value,
      totalComensais: totalComensais.value,
    };
  }

  private events: any = {
    click: () => DialogService.hide(),
  };

  // FUNÇÃO DE CONFIRMAR ALTERAÇÃO COMENSAIS
  public confirmaAlteraComensais() {
    const formTurno = Metadata.getInstance<Form>('formAlteraTurnos');
    const formPontoEntrega = Metadata.getInstance<Form>('formAlteraPontoEntrega');
    const grid: Grid = Metadata.getInstance('gridAlteracaoCardapio');
    const currentRowGrid = grid.datasource.currentRow;
    const rowTurno = formTurno.value;
    const rowPontoEntrega = formPontoEntrega.value;
    const quantidadeTurnos = [];
    const quantidadePontosEntrega = [];
    this.configuraCamposComensais();
    const { rowComensais } = this;
    Object.keys(rowTurno).forEach((key) => {
      const lastChar = key.slice(-1);
      rowTurno[lastChar] = rowTurno[key];
      if (/[a-zA-Z]/.test(key)) {
        quantidadeTurnos.push(rowTurno[key]);
      }
    });
    Object.keys(rowPontoEntrega).forEach((key) => {
      quantidadePontosEntrega.push(rowPontoEntrega[key]);
    });
    const somaTurnos = quantidadeTurnos.reduce((acumulador, valorAtual) => acumulador + valorAtual, 0);
    if ((Object.keys(rowTurno).length > 0) && (somaTurnos !== rowComensais.totalComensais)) {
      DialogService.show({
        name: 'error-dialog',
        type: 'error',
        title: 'plaNext::erro',
        text: 'plaNext::erroSomaComensaisTurnos',
      });
      return;
    }
    const valoresUpdate = {
      rowComensais,
      turnos: rowTurno,
      row: currentRowGrid,
    };
    LoadingService.show();
    Http.post('/alteraComensaisTurno', { requestType: 'Row', valoresUpdate }).then(() => {
      this.configureMenuChanged(true);
    }).catch((error: string) => {
      AlertService.show(this.configuraAlert('alertError', error, 'error'));
    }).finally(() => {
      this.fechaModalAlteraCellComensais();
      LoadingService.hide();
    });
    // const somaPontosEntrega = quantidadePontosEntrega.reduce((acumulador, valorAtual) => acumulador + valorAtual, 0);
    // const somaPontosEntregaString = `${somaPontosEntrega}`;
  }

  public confirmChangedDish() {
    const formDish = Metadata.getInstance<Form>('modalChangedDish');
    const gridMenu : GridEditable = Metadata.getInstance('tableCardapio');
    const { currentRow } = gridMenu.datasource;
    const dinerMenu = currentRow.COMENSAISCARDAPIO;
    if (dinerMenu && dinerMenu !== null && dinerMenu !== undefined) {
      if (formDish.validate()) {
        const grid: Grid = Metadata.getInstance('gridAlteracaoCardapio');
        const currentRowGrid = grid.datasource.currentRow;
        const rowDish = formDish.value;
        rowDish.VRPECONSCARD = rowDish.totalComensaisPrato;
        const dtMesCardapio = currentRowGrid.DTMESCARD.substring(2, 10);
        const diaCardapio = currentRow.DIASCARDAPIO.substring(4);
        rowDish.DTDIACARD = diaCardapio + dtMesCardapio;
        currentRowGrid.DTDIACARD = rowDish.DTDIACARD;
        rowDish.NRSEQPRATCAR = this.columnDish;
        const valoresUpdate = {
          dish: rowDish,
          row: currentRowGrid,
        };
        LoadingService.show();
        Http.post('/changedDish', { requestType: 'Row', valoresUpdate }).then(() => {
          this.configureMenuChanged(true);
        }).catch((error: string) => {
          AlertService.show(this.configuraAlert('alertError', error, 'error'));
        }).finally(() => {
          AlertService.show(this.configuraAlert('alert', 'plaNext::atualizadoComSucesso', 'success'));
          this.fechaModalAlteraPratos();
          LoadingService.hide();
        });
      }
    } else {
      AlertService.show(this.configuraAlert('alertError', 'Não há comensais cadastrados para este dia!', 'error'));
    }
  }

  public async criaModalAlteraPratos() {
    this.modalAlteraPratos = await ModalService.createFromJson('/modals/alteraCellPratos', true);
  }

  private rowPratos = [];

  private columnDish = '';

  // FUNÇÃO QUE VERIFICA ONDE FOI REALIZADO O CLICK DA CELULA
  public abreCellClick({ column, row }: any) {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const rowGridChangedMenu = grid.datasource.currentRow;
    if (!this.checkModeApproval() && rowGridChangedMenu.IDSITUACAO === 'E') {
      if (column.name === 'COMENSAISCARDAPIO') {
        this.configuraCelulaComensais(row);
      } else if (column.name === 'DIASCARDAPIO') {
        // Não abre nenhum modal.
      } else {
        this.abreModalAlteraPratos();
        if (this.rowPratos.length > 0) {
          this.rowPratos = [];
        }
        this.rowPratos.push(row);
        this.getCodPrato(column.name, row);
        this.getComensalPrato(column.name, row);
        this.columnDish = column.componentProps.text;
      }
    }
  }

  // MODAL DE PRATOS - CONFIGURAÇÃO E CRIAÇÃO
  public abreModalAlteraPratos() {
    this.criaModalAlteraPratos().then(() => {
      this.modalAlteraPratos.show();
    });
  }

  private codPratoRecorrente = '';

  private getCodPrato(columnName, row) {
    if (Object.prototype.hasOwnProperty.call(row, columnName)) {
      const chaveCD = `${columnName}CD`;
      this.codPratoRecorrente = row[chaveCD];
    }
    return null;
  }

  private comPratoRecorrente = '';

  private getComensalPrato(columnName, row) {
    const valorOriginal = row[columnName];
    const valorNumerico = parseFloat(valorOriginal);
    let rowComensais = row[`${columnName}CM`];
    rowComensais = valorNumerico.toString();
    this.comPratoRecorrente = rowComensais;
  }

  public configuraModalAlteraPratos() {
    const selectPratos = Metadata.getInstance<Select>('selectPratoModal');
    const totalComensais = Metadata.getInstance<NumberInput>('totalComensaisPrato');
    const nrOrg = localStorage.getItem('NRORG');
    const rowPratosArray = this.rowPratos[0];
    const filter = {
      NRORG: nrOrg,
      CDNVPRATO: '4', // verificar utilização do nvl do prato
      CDFILIAL: rowPratosArray.CDFILIAL,
    };
    selectPratos.datasource.filter = filter;
    selectPratos.datasource.get();
    const valorCdPratoRecorrente = this.codPratoRecorrente;
    const valorComPratoRecorrente = this.comPratoRecorrente;
    totalComensais.value = valorComPratoRecorrente;
    const value = {
      CDPRATO: valorCdPratoRecorrente,
    };
    selectPratos.value = value;
  }

  public fechaModalAlteraPratos() {
    this.modalAlteraPratos.hide();
    this.modalAlteraPratos.destroy();
  }

  public async createCompleteOrderModal() {
    this.completeOrderModal = await ModalService.createFromJson('/modals/completeOrder', true);
  }

  public openCompleteOrderModal() {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const row = grid.datasource.currentRow;
    if (Object.keys(row).length > 0) {
      if (row.IDSITUACAO === 'E') {
        this.completeOrderModal.show();
      } else {
        AlertService.show(this.configuraAlert('alertError', 'plaNext::orderInProgress', 'error'));
      }
    } else {
      AlertService.show(this.configuraAlert('alertError', 'plaNext::nenhumRegSelecionado', 'error'));
    }
  }

  public completeOrder() {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const row = grid.datasource.currentRow;
    const form = Metadata.getInstance<Form>('formCompleteOrder');
    const { extraRequest } = form.value;
    const sendObject = {
      CURRENTROW: row,
      EXTRAREQUEST: extraRequest,
    };
    LoadingService.show();
    Http.post('/completeOrder', {
      requestType: 'Row',
      row: sendObject,
    }).then(() => {
      this.configuraGridAlteracao();
      AlertService.show(this.configuraAlert('alert', 'plaNext::completeOrderSuccess', 'success'));
    }).catch((e) => {
      let { error } = e.response.data;
      error = error.substr(11);
      AlertService.show(this.configuraAlert('alertError', error, 'error'));
    }).finally(() => {
      LoadingService.hide();
      this.completeOrderModal.hide();
    });
  }

  confirmRequestAgain() {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const row = grid.datasource.currentRow;
    if (Object.keys(row).length > 0) {
      if (row.IDSITUACAO === 'R') {
        DialogService.show({
          name: 'dialogCompleteOrder',
          type: 'warning',
          title: 'plaNext::solicitacao',
          text: 'plaNext::confirmRequestChanged',
          buttons: [
            {
              name: 'yes',
              component: 'ZdButton',
              label: 'plaNext::sim',
              events: {
                click: () => this.requestAgain(row),
              },
              outline: true,
            },
            {
              name: 'no',
              component: 'ZdButton',
              label: 'plaNext::nao',
              events: {
                click: () => DialogService.hide(),
              },
            },
          ],
        });
      } else {
        AlertService.show(this.configuraAlert('alertError', 'plaNext::requestRejected', 'error'));
      }
    } else {
      AlertService.show(this.configuraAlert('alertError', 'plaNext::nenhumRegSelecionado', 'error'));
    }
  }

  public requestAgain(currentRow) {
    LoadingService.show();
    Http.post('/requestAgain', {
      requestType: 'Row',
      row: currentRow,
    }).then(() => {
      this.configuraGridAlteracao();
      AlertService.show(this.configuraAlert('alert', 'plaNext::requestSuccessfullySubmitted', 'success'));
    }).catch((e) => {
      let { error } = e.response.data;
      error = error.substr(11);
      AlertService.show(this.configuraAlert('alertError', error, 'error'));
    }).finally(() => {
      LoadingService.hide();
      DialogService.hide();
    });
  }

  public async createApproveDisapproveModal() {
    this.approveDisapproveModal = await ModalService.createFromJson('/modals/approveDisapprove', true);
  }

  public openApproveDisapproveModal() {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const row = grid.datasource.currentRow;
    if (Object.keys(row).length > 0) {
      if (row.IDSITUACAO === 'C') {
        const pageApproval = this.checkModeApproval();
        const apGO = row.APPROVALGO;
        if (!pageApproval && row.IDSOLEXTRA === 'N') {
          AlertService.show(this.configuraAlert('alertError', 'plaNext::noExtraRequest', 'error'));
        } else if (pageApproval && (apGO === null || apGO === ' ') && row.IDSOLEXTRA === 'N') {
          this.approveDisapproveModal.show();
        } else if ((!pageApproval && (apGO === null || apGO === ' ')) || (pageApproval && (apGO !== null && apGO !== ' '))) {
          this.approveDisapproveModal.show();
        } else if (pageApproval && (apGO === null || apGO === ' ') && row.IDSOLEXTRA === 'S') {
          AlertService.show(this.configuraAlert('alertError', 'plaNext::approvalRequiredGO', 'error'));
        } else if (!pageApproval && (apGO !== null && apGO !== ' ')) {
          AlertService.show(this.configuraAlert('alertError', 'plaNext::existApproveGO', 'error'));
        }
      } else {
        AlertService.show(this.configuraAlert('alertError', 'plaNext::completedOrders', 'error'));
      }
    } else {
      AlertService.show(this.configuraAlert('alertError', 'plaNext::nenhumRegSelecionado', 'error'));
    }
  }

  get titleTextPage() {
    let titleText = '';
    if (this.checkModeApproval()) {
      titleText = 'plaNext::pedidoAlteracaoCardapio';
    } else {
      titleText = 'plaNext::pedidoAlteracaoCardapioGO';
    }
    return titleText;
  }

  get titleText() {
    let titleText = '';
    if (this.checkModeApproval()) {
      titleText = 'plaNext::approveDisapprove';
    } else {
      titleText = 'plaNext::approveDisapproveGO';
    }
    return titleText;
  }

  public confirmApproveDisapprove({ component }) {
    const valueFormJustification = Metadata.getInstance<Textarea>('justification').value;
    let textConfirm = '';
    let approve = '';
    if (component.name === 'approve') {
      textConfirm = 'plaNext::approve';
      approve = 'A';
    } else if (valueFormJustification !== null && valueFormJustification.length > 0) {
      textConfirm = 'plaNext::disapprove';
      approve = 'R';
    } else {
      AlertService.show(this.configuraAlert('alertError', 'plaNext::fillJustification', 'error'));
      return;
    }
    DialogService.show({
      name: 'dialogCompleteOrder',
      type: 'warning',
      text: textConfirm,
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'plaNext::sim',
          events: {
            click: () => this.approveDisapprove(approve, valueFormJustification),
          },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'plaNext::nao',
          events: {
            click: () => DialogService.hide(),
          },
        },
      ],
    });
  }

  public approveDisapprove(approve, valueFormJustification) {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const row = grid.datasource.currentRow;
    const sendObject = {
      CURRENTROW: row,
      APPROVE: approve,
      JUSTIFICATION: valueFormJustification,
      APPROVALPAGE: this.checkModeApproval(),
    };
    LoadingService.show();
    Http.post('/approveDisapprove', {
      requestType: 'Row',
      row: sendObject,
    }).then(() => {
      this.configuraGridAlteracao();
      AlertService.show(this.configuraAlert('alert', 'plaNext::requestSuccessfullySubmitted', 'success'));
    }).catch((e) => {
      let { error } = e.response.data;
      error = error.substr(11);
      AlertService.show(this.configuraAlert('alertError', error, 'error'));
    }).finally(() => {
      LoadingService.hide();
      DialogService.hide();
      this.approveDisapproveModal.hide();
    });
  }
}
