package com.javaseleniumtemplate.pages;

import com.javaseleniumtemplate.bases.PageBase;
import org.openqa.selenium.By;
import org.openqa.selenium.Keys;
import org.openqa.selenium.interactions.Actions;
import org.testng.Assert;

public class PadraoCicloPage extends PageBase {

    By botaoMenu = By.xpath("//div[@class='bars']");
    By campoPesquisar = By.xpath("//section[@class='zh-container-search-menu']//input[@type='text']");
    By selecionarOpcaoPadraoCiclo = By.xpath("//section[@class='something-else']//span[text() = 'Padrão de Ciclo']");

    By botaoAdicionar = By.xpath("//div[@class=\"zh-footer-center\"]//div[@class=\"zh-test-box\"]");
    By botaoSalvar = By.xpath("//div[@class=\"zh-footer-right\"]");
    By botaoCancelar = By.xpath("//div[@class=\"zh-footer-left\"]");
    By botaoExcluir = By.xpath("//div[@class=\"zh-footer-center\"]//div[@class=\"zh-test-box\"]");
    By botaoSim = By.xpath("//section[@class=\"zh-container-alert\"]//button[1]");
    By botaoOk = By.xpath("//section[@class=\"zh-container-alert\"]//button[@type=\"submit\"]");

    By campoCodigo = By.id("CDPADCICLO");
    By campoNome = By.id("NMPADCICLO");
    By mensagemObrigatorioCodigo = By.xpath("//div[@data-zh-id=\"4478476822839877127\"]//div[@class=\"zh-validation\"]/span");
    By mensagemObrigatorioNome = By.xpath("//div[@data-zh-id=\"4478476540327564128\"]//div[@class=\"zh-validation\"]/span");

    By checkBoxSegundoPadraoTelaInicial = By.xpath("//div[@id=\"grid-44784762741154281463\"]//div[@class=\"body-content\"]/div[2]/div[@data-zh-field-name=\"__CHECKBOX\"]");
    By checkBoxTerceiroPadraoTelaInicial = By.xpath("//div[@id=\"grid-44784762741154281463\"]//div[@class=\"body-content\"]/div[3]/div[@data-zh-field-name=\"__CHECKBOX\"]");
    By codigoTerceiroPadraoTelaInicial = By.xpath("//div[@id=\"grid-44784762741154281463\"]//div[@class=\"body-content\"]/div[3]/div[2]/span");
    By nomeTerceiroPadraoTelaInicial = By.xpath("//div[@id=\"grid-44784762741154281463\"]//div[@class=\"body-content\"]/div[3]/div[3]/span");

    By mensagemSucesso = By.xpath("//*[@class=\"zh-notification-messages\"]//span[@class=\"notification-message-text\"]");
    By mensagemAlerta = By.xpath("//section[@class=\"zh-container-alert\"]//div[@class=\"zh-alert-scroll\"]/p");

    public void acessarTela() {
        tempoParaAguardarTelaInicial();
        waitForElement(botaoMenu);
        click(botaoMenu);
        sendKeys(campoPesquisar, "Padrão de Ciclo");
        click(selecionarOpcaoPadraoCiclo);
    }

    public void adicionarPadrao() {
        tempo();
        click(botaoAdicionar);
        sendKeys(campoCodigo, "3");
        sendKeys(campoNome, "AUTOMAÇÃO");
        tempo();
        click(botaoSalvar);

        String mensagem = getText(mensagemSucesso);
        String mensagemEsperada = "Salvo com sucesso.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificação da mensagem de sucesso, após adicionar o padrão de ciclo");
    }

    public void verificarPadraoCicloAdd() {
        //valores encontrados
        String codigo = getText(codigoTerceiroPadraoTelaInicial);
        String nome = getText(nomeTerceiroPadraoTelaInicial);

        Assert.assertEquals(codigo, "00003", "Verificação do código do padrão de ciclo adicionado");
        Assert.assertEquals(nome, "AUTOMAÇÃO", "Verificação do nome do padrão de ciclo adicionado");
    }

    public void editarPadraoClico() {
        Actions actions = new Actions(driver);
        actions.moveToElement(driver.findElement(nomeTerceiroPadraoTelaInicial)).click().build().perform();
        tempo();
        actions.sendKeys(Keys.BACK_SPACE).build().perform();
        tempo();
        actions.sendKeys("AUTOMAÇÃO EDITADA").build().perform();
        tempo();
        actions.sendKeys(Keys.ENTER).build().perform();
        click(botaoSalvar);

        String mensagem = getText(mensagemSucesso);
        String mensagemEsperada = "Salvo com sucesso.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificação da mensagem de sucesso, após editar o padrão de ciclo");

        String nome = getText(nomeTerceiroPadraoTelaInicial);
        String nomeEsperado = "AUTOMAÇÃO EDITADA";
        Assert.assertEquals(nome, nomeEsperado, "Verificação do nome do padrão de ciclo, após a edição");
    }

    public void excluirPadraoCiclo() {
        tempo();
        click(checkBoxTerceiroPadraoTelaInicial);
        click(botaoExcluir);

        String mensagem = getText(mensagemAlerta);
        String mensagemEsperada = "Deletar o(s) registro(s)?";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificação da mensagem exibida na tela após click no botão excluir");
        click(botaoSim);

        mensagem = getText(mensagemSucesso);
        mensagemEsperada = "Excluído com sucesso.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificação da mensagem de sucesso, após excluir o padrão de ciclo");
    }

    public void verificarCamposObrigatorios() {
        click(botaoAdicionar);
        click(botaoSalvar);

        //mensagem de campo obrigatório
        String codigo = getText(mensagemObrigatorioCodigo);
        String nome = getText(mensagemObrigatorioNome);

        String mensagemEsperada = "Campo obrigatório";
        Assert.assertEquals(codigo, mensagemEsperada, "Verificação da mensagem de campo obrigatório, no campo código");
        Assert.assertEquals(nome, mensagemEsperada, "Verificação da mensagem de campo obrigatório, no campo nome");
        click(botaoCancelar);
    }

    public void tentarExcluirPadraoComRegFilho() {
        click(checkBoxSegundoPadraoTelaInicial);
        click(botaoExcluir);
        click(botaoSim);

        String mensagem = getText(mensagemAlerta);
        String mensagemEsperada = "Exclusão não permitida! O registro possui registros filhos.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificação da mensagem exibida na tela após click no botão excluir um padrão de ciclo que possui registros filhos");
        click(botaoOk);
        tempo();
        click(checkBoxSegundoPadraoTelaInicial);
    }

    public void tentarExcluriDoisPadroes() {
        click(checkBoxSegundoPadraoTelaInicial);
        click(checkBoxTerceiroPadraoTelaInicial);
        click(botaoExcluir);
        click(botaoSim);

        String mensagem = getText(mensagemAlerta);
        String mensagemEsperada = "Exclusão não permitida! O registro possui registros filhos.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificação da mensagem exibida na tela após click no botão excluir um padrão de ciclo que possui registros filhos");
        click(botaoOk);
        click(checkBoxSegundoPadraoTelaInicial);
        click(checkBoxTerceiroPadraoTelaInicial);
    }

    public void tentarAddPadraoComCodigoExistente() {
        click(botaoAdicionar);
        sendKeys(campoCodigo, "1");
        sendKeys(campoNome,"AUTOMAÇÃO");
        click(botaoSalvar);

        String mensagem = getText(mensagemAlerta);
        String mensagemEsperada = "Impossível adicionar. Registro já existe.";
        Assert.assertEquals(mensagem, mensagemEsperada, "Verificação da mensagem exibida na tela após tentar adicionar um padrão de ciclo com código existente");
        click(botaoOk);
        click(botaoCancelar);
    }

}
